@@ -1,7 +1,7 @@
 """
 .. note::
 
-    * Aiogoogle is an async-framework agnosting library. As a result, adding file IO OR credentials storage capabilities for user credentials would add unneeded complexity. 
+    * Aiogoogle is an async-framework agnosting library. As a result, adding file IO OR credentials storage capabilities for user credentials would add unneeded complexity.
 
     * Credentials are an instance of ``dict`` and are guaranteed to only contain JSON types (str, number, array, JSONSCHEMA datetime and ISO8601 datetime, etc) to make them easily serializable.
 """
@@ -156,9 +156,9 @@ class Oauth2Manager:
         Example:
 
             * response_types_supported
-            
+
             * scopes_supported
-            
+
             * claims_supported
         """
         try:
@@ -190,7 +190,7 @@ class Oauth2Manager:
 
         Unless this test is failing:
 
-        aiogoogle.tests.integ_online.latest_test_latest_openid_configs(), You shouldn't really need to use this. 
+        aiogoogle.tests.integ_online.latest_test_latest_openid_configs(), You shouldn't really need to use this.
         """
         req = Request("GET", url=OPENID_CONFIGS_DISCOVERY_DOC_URL)
         self.openid_configs = await self._send_request(req)
@@ -229,7 +229,7 @@ class Oauth2Manager:
 
         Returns:
 
-            bool: 
+            bool:
         """
         client_creds = client_creds or self.client_creds
         if (
@@ -253,7 +253,7 @@ class Oauth2Manager:
         response_type=AUTH_CODE_RESPONSE_TYPE,
         scopes=None,
     ) -> (str):
-        """ 
+        """
         First step of OAuth2 authoriztion code flow. Creates an OAuth2 authorization URI.
 
         Arguments:
@@ -271,27 +271,27 @@ class Oauth2Manager:
                 * Optional
 
                 * Overrides the list of scopes specified in client creds
-            
+
             state (str): A CSRF token
 
                 * Optional
 
                 * Specifies any string value that your application uses to maintain state between your authorization request and the authorization server's response.
-                
+
                 * The server returns the exact value that you send as a name=value pair in the hash (#) fragment of the redirect_uri after the user consents to or denies your application's access request.
 
                 * You can use this parameter for several purposes, such as:
-                
+
                     * Directing the user to the correct resource in your application
-                    
+
                     * Sending nonces
-                    
+
                     * Mitigating cross-site request forgery.
 
                 * If no state is passed, this method will generate and add a secret token to ``user_creds['state']``.
-                    
+
                 * Since your redirect_uri can be guessed, using a state value can increase your assurance that an incoming connection is the result of an authentication request.
-                
+
                 * If you generate a random string or encode the hash of a cookie or another value that captures the client's state, you can validate the response to additionally ensure that the request and response originated in the same browser, providing protection against attacks such as cross-site request forgery.
 
             access_type (str): Indicates whether your application can refresh access tokens when the user is not present at the browser. Options:
@@ -303,33 +303,33 @@ class Oauth2Manager:
                 * ``"offline"`` Choose this for a refresheable/long-term access token
 
             include_granted_scopes (bool):
-            
+
                 * Optional
 
                 * Enables applications to use incremental authorization to request access to additional scopes in context.
-                
+
                 * If you set this parameter's value to ``True`` and the authorization request is granted, then the new access token will also cover any scopes to which the user previously granted the application access.
 
             login_hint (str):
-            
+
                 * Optional
-                
+
                 * If your application knows which user is trying to authenticate, it can use this parameter to provide a hint to the Google Authentication Server.
-                
+
                 * The server uses the hint to simplify the login flow either by prefilling the email field in the sign-in form or by selecting the appropriate multi-login session.
 
                 * Set the parameter value to an email address or sub identifier, which is equivalent to the user's Google ID.
-                
+
                 * This can help you avoid problems that occur if your app logs in the wrong user account.
 
             prompt (str):
 
                 * Optional
-                
+
                 * A space-delimited, case-sensitive list of prompts to present the user.
-                
+
                 * If you don't specify this parameter, the user will be prompted only the first time your app requests access.
-                
+
                 * Possible values are:
 
                     * ``None`` : Default: Do not display any authentication or consent screens. Must not be specified with other values.
@@ -359,19 +359,19 @@ class Oauth2Manager:
             * For a list of all of Google's available scopes: https://developers.google.com/identity/protocols/googlescopes
 
             * It is recommended that your application requests access to authorization scopes in context whenever possible.
-            
+
             * By requesting access to user data in context, via incremental authorization, you help users to more easily understand why your application needs the access it is requesting.
 
         Warning:
 
             * When listening for a callback after redirecting a user to the URL returned from this method, take the following into consideration:
-            
+
                 * If your response endpoint renders an HTML page, any resources on that page will be able to see the authorization code in the URL.
-                
+
                 * Scripts can read the URL directly, and the URL in the Referer HTTP header may be sent to any or all resources on the page.
 
                 * Carefully consider whether you want to send authorization credentials to all resources on that page (especially third-party scripts such as social plugins and analytics).
-                
+
                 * To avoid this issue, it's recommend that the server first handle the request, then redirect to another URL that doesn't include the response parameters.
 
         Example:
@@ -431,9 +431,9 @@ class Oauth2Manager:
         Arguments:
 
             grant (str):
-            
-                * Aka: "code". 
-                
+
+                * Aka: "code".
+
                 * The code received at your redirect URI from the auth callback
 
             client_creds (aiogoogle.auth.creds.ClientCreds):
@@ -504,9 +504,9 @@ class Oauth2Manager:
     def authorized_for_method(method, user_creds) -> bool:
         """
         Checks if oauth2 user_creds dict has sufficient scopes for a method call.
-        
-        .. note:: 
-        
+
+        .. note::
+
             This method doesn't check whether creds are refreshed or valid.
 
         e.g.
@@ -514,9 +514,9 @@ class Oauth2Manager:
             **Correct:**
 
             .. code-block:: python3
-        
+
                 is_authorized = authorized_for_method(youtube.video.list)
-            
+
             **NOT correct:**
 
             .. code-block:: python3
@@ -595,7 +595,7 @@ class Oauth2Manager:
         Checks if user_creds expired
 
         Arguments:
-        
+
             user_creds (aiogoogle.auth.creds.UserCreds): User Credentials
 
         Returns:
@@ -608,7 +608,7 @@ class Oauth2Manager:
     async def refresh(self, user_creds, client_creds=None):
         """
         Refreshes user_creds
-        
+
         Arguments:
 
             user_creds (aiogoogle.auth.creds.UserCreds): User Credentials with ``refresh_token`` item
@@ -617,6 +617,7 @@ class Oauth2Manager:
 
         Returns:
 
+            bool: If the token is refreshed or not
             aiogoogle.creds.UserCreds: Refreshed user credentials
 
         Raises:
@@ -624,12 +625,16 @@ class Oauth2Manager:
             aiogoogle.excs.AuthError: Auth Error
         """
         client_creds = client_creds or self.client_creds
+
+        if user_creds.get("expires_at") and not self.is_expired(user_creds):
+            return False, user_creds
+
         request = self._build_refresh_request(user_creds, client_creds)
         json_res = await self._send_request(request)
         final_user_creds = self._build_user_creds_from_res(json_res)
         if not final_user_creds.get('refresh_token'):
             final_user_creds['refresh_token'] = user_creds.get('refresh_token')
-        return final_user_creds
+        return True, final_user_creds
 
     def _build_refresh_request(self, user_creds, client_creds):
         data = dict(
@@ -682,21 +687,21 @@ class OpenIdConnectManager(Oauth2Manager):
     async def get_user_info(self, user_creds):
         """
         https://developers.google.com/+/web/api/rest/openidconnect/getOpenIdConnect
-    
+
         People: getOpenIdConnect
-    
+
         Get user information after performing an OpenID connect flow.
-    
+
         Use this method instead of people.get (Google+ API) when you need the OpenID Connect format.
-    
-        This method is not discoverable nor is it in the Google API client libraries. 
-        
+
+        This method is not discoverable nor is it in the Google API client libraries.
+
         To learn more, see OpenID Connect for sign-in. https://developers.google.com/+/web/api/rest/openidconnect/index.html
-    
+
         Example:
-    
+
             ::
-    
+
                 >>> await get_user_info(user_creds)
                 {
                     "kind": "plus#personOpenIdConnect",
@@ -723,13 +728,13 @@ class OpenIdConnectManager(Oauth2Manager):
 
     async def get_token_info_jwt(self, user_creds):
         """ get token info using id_token_jwt instead of access_token ``self.get_token_info``
-        
+
         Arguments:
-        
+
             user_creds (aiogoogle.auth.creds.UserCreds): user_creds with id_token_jwt item
-            
+
         Returns:
-        
+
             dict: Information about the token
         """
         req = self.oauth2_api.tokeninfo(id_token=user_creds.get("id_token_jwt"))
@@ -758,7 +763,7 @@ class OpenIdConnectManager(Oauth2Manager):
         response_type=AUTH_CODE_RESPONSE_TYPE,
         scopes=None,
     ):
-        """ 
+        """
         First step of OAuth2 authoriztion code flow. Creates an OAuth2 authorization URI.
 
         Arguments:
@@ -788,15 +793,15 @@ class OpenIdConnectManager(Oauth2Manager):
                 * Optional
 
                 * An ASCII string value for specifying how the authorization server displays the authentication and consent user interface pages.
-                
+
                 * The following values are specified, and accepted by the Google servers, but do not have any effect on its behavior:
-                
+
                     * ``page``
-                    
+
                     * ``popup``
-                    
+
                     * ``touch``
-                    
+
                     * ``wap``
 
             state (str): A CSRF token
@@ -804,21 +809,21 @@ class OpenIdConnectManager(Oauth2Manager):
                 * Optional
 
                 * Specifies any string value that your application uses to maintain state between your authorization request and the authorization server's response.
-                
+
                 * The server returns the exact value that you send as a name=value pair in the hash (#) fragment of the redirect_uri after the user consents to or denies your application's access request.
 
                 * You can use this parameter for several purposes, such as:
-                
+
                     * Directing the user to the correct resource in your application
-                    
+
                     * Sending nonces
-                    
+
                     * Mitigating cross-site request forgery.
 
                 * If no state is passed, this method will generate and add a secret token to ``user_creds['state']``.
-                    
+
                 * Since your redirect_uri can be guessed, using a state value can increase your assurance that an incoming connection is the result of an authentication request.
-                
+
                 * If you generate a random string or encode the hash of a cookie or another value that captures the client's state, you can validate the response to additionally ensure that the request and response originated in the same browser, providing protection against attacks such as cross-site request forgery.
 
             access_type (str): Indicates whether your application can refresh access tokens when the user is not present at the browser. Options:
@@ -830,19 +835,19 @@ class OpenIdConnectManager(Oauth2Manager):
                 * ``"offline"`` Choose this for a refresheable/long-term access token
 
             include_granted_scopes (bool):
-            
+
                 * Optional
 
                 * Enables applications to use incremental authorization to request access to additional scopes in context.
-                
+
                 * If you set this parameter's value to ``True`` and the authorization request is granted, then the new access token will also cover any scopes to which the user previously granted the application access.
 
             login_hint (str):
-            
+
                 * Optional
-                
+
                 * If your application knows which user is trying to authenticate, it can use this parameter to provide a hint to the Google Authentication Server.
-                
+
                 * The server uses the hint to simplify the login flow either by prefilling the email field in the sign-in form or by selecting the appropriate multi-login session.
 
                 * Set the parameter value to an email address or sub identifier, which is equivalent to the user's Google ID.
@@ -852,11 +857,11 @@ class OpenIdConnectManager(Oauth2Manager):
             prompt (str):
 
                 * Optional
-                
+
                 * A space-delimited, case-sensitive list of prompts to present the user.
-                
+
                 * If you don't specify this parameter, the user will be prompted only the first time your app requests access.
-                
+
                 * Possible values are:
 
                     * ``None`` : Default: Do not display any authentication or consent screens. Must not be specified with other values.
@@ -870,11 +875,11 @@ class OpenIdConnectManager(Oauth2Manager):
                 * Optional
 
                 * openid.realm is a parameter from the OpenID 2.0 protocol.
-                
+
                 * It is used in OpenID 2.0 requests to signify the URL-space for which an authentication request is valid.
-                
+
                 * Use openid.realm if you are migrating an existing application from OpenID 2.0 to OpenID Connect.
-                
+
                 * For more details, see Migrating off of OpenID 2.0. https://developers.google.com/identity/protocols/OpenID2Migration
 
             hd (str):
@@ -882,15 +887,15 @@ class OpenIdConnectManager(Oauth2Manager):
                 * Optional
 
                 * The hd (hosted domain) parameter streamlines the login process for G Suite hosted accounts.
-                
+
                 * By including the domain of the G Suite user (for example, mycollege.edu), you can indicate that the account selection UI should be optimized for accounts at that domain.
-                
+
                 * To optimize for G Suite accounts generally instead of just one domain, use an asterisk: hd=*.
 
                 * Don't rely on this UI optimization to control who can access your app, as client-side requests can be modified.
-                
+
                 * Be sure to validate that the returned ID token has an hd claim value that matches what you expect (e.g. mycolledge.edu).
-                
+
                 * Unlike the request parameter, the ID token claim is contained within a security token from Google, so the value can be trusted.
 
             response_type (str):
@@ -916,19 +921,19 @@ class OpenIdConnectManager(Oauth2Manager):
             * For a list of all of Google's available scopes: https://developers.google.com/identity/protocols/googlescopes
 
             * It is recommended that your application requests access to authorization scopes in context whenever possible.
-            
+
             * By requesting access to user data in context, via incremental authorization, you help users to more easily understand why your application needs the access it is requesting.
 
         Warning:
 
             * When listening for a callback after redirecting a user to the URL returned from this method, take the following into consideration:
-            
+
                 * If your response endpoint renders an HTML page, any resources on that page will be able to see the authorization code in the URL.
-                
+
                 * Scripts can read the URL directly, and the URL in the Referer HTTP header may be sent to any or all resources on the page.
 
                 * Carefully consider whether you want to send authorization credentials to all resources on that page (especially third-party scripts such as social plugins and analytics).
-                
+
                 * To avoid this issue, it's recommend that the server first handle the request, then redirect to another URL that doesn't include the response parameters.
 
         Example:
@@ -1002,9 +1007,9 @@ class OpenIdConnectManager(Oauth2Manager):
         Arguments:
 
             grant (str):
-            
-                * Aka: "code". 
-                
+
+                * Aka: "code".
+
                 * The code received at your redirect URI from the auth callback
 
             client_creds (aiogoogle.auth.creds.ClientCreds):
@@ -1233,7 +1238,7 @@ class ServiceAccountManager:
         4. Compute Engine default credentials
 
         Returns:
-        
+
             None
         '''
         if self.creds.get('private_key'):
@@ -1256,7 +1261,7 @@ class ServiceAccountManager:
                         url=GCE_METADATA_IP_ROOT,
                         headers=GCE_METADATA_HEADERS,
                         timeout=GCE_METADATA_DEFAULT_TIMEOUT,
-                        _verify_ssl=False
+                        _verify_ssl=False,
                     ), full_res=True)
                 except Exception:
                     raise RuntimeError(
@@ -1293,7 +1298,7 @@ class ServiceAccountManager:
             additional_claims = creds.pop('additional_claims')
         except KeyError:
             additional_claims = None
-        
+
         google_auth_lib_creds = service_account.Credentials.from_service_account_info(
             info=creds,
             scopes=scopes,
@@ -1311,7 +1316,7 @@ class ServiceAccountManager:
                 data=parse.urlencode({
                     "assertion": google_auth_lib_creds._make_authorization_grant_assertion(),
                     "grant_type": JWT_GRANT_TYPE
-                }).encode("utf-8")
+                }).encode("utf-8"),
             ))
 
         if not json_res.get('access_token'):
@@ -1325,11 +1330,11 @@ class ServiceAccountManager:
         Ensures that there's an unexpired access token.
 
         Returns:
+            bool: If the token is refreshed or not.
 
-            None
         '''
         if self._access_token and not _is_expired(self._expires_at):
-            return
+            return False
 
         if self._creds_source == 'key_file':
             await self._get_oauth2_authorization_grant()
@@ -1340,3 +1345,4 @@ class ServiceAccountManager:
                 'No service account creds found.'
                 'Please provide service account credentials or call self.detect_default_creds first'
             )
+        return True
