@@ -1,13 +1,20 @@
+from __future__ import annotations
+
 __all__ = ["Aiogoogle"]
 
 from contextvars import ContextVar
-from typing import Optional
+from typing import TYPE_CHECKING, Any, Literal, Optional, Type
 
 from .resource import GoogleAPI
 from .auth.managers import Oauth2Manager, ApiKeyManager, OpenIdConnectManager, ServiceAccountManager
 from .sessions.aiohttp_session import AiohttpSession
 from .data import DISCOVERY_SERVICE_V1_DISCOVERY_DOC
+from .excs import HTTPError
 
+if TYPE_CHECKING:
+    from .auth.creds import ApiKey, ClientCreds, ServiceAccountCreds, UserCreds
+    from .models import Request, Response
+    from .sessions.abc import AbstractSession
 
 # Discovery doc reference https://developers.google.com/discovery/v1/reference/apis
 
@@ -27,7 +34,7 @@ class Aiogoogle:
         2. Aiogoogle's OAuth2 manager
         3. Aiogoogle's API key manager
         4. Aiogoogle's OpenID Connect manager
-        5. Aiogoogle's service account manager 
+        5. Aiogoogle's service account manager
         6. One of Aiogoogle's implementations of a session object
 
     Arguments:
@@ -35,28 +42,28 @@ class Aiogoogle:
         session_factory (aiogoogle.sessions.abc.AbstractSession): AbstractSession Implementation. Defaults to ``aiogoogle.sessions.aiohttp_session.AiohttpSession``
 
         api_key (aiogoogle.auth.creds.ApiKey): Google API key
-        
-        user_creds (aiogoogle.auth.creds.UserCreds): OAuth2 cser credentials 
+
+        user_creds (aiogoogle.auth.creds.UserCreds): OAuth2 user credentials
 
         client_creds (aiogoogle.auth.creds.ClientCreds): OAuth2 client credentials
 
         service_account_creds (aiogoogle.auth.creds.ServiceAccountCreds): Service account credentials
-        
-    Note: 
-    
+
+    Note:
+
         In case you want to instantiate a custom session with initial parameters, you can pass an anonymous factory. e.g. ::
-        
+
             >>> sess = lambda: Session(your_custome_arg, your_custom_kwarg=True)
             >>> aiogoogle = Aiogoogle(session_factory=sess)
     """
 
     def __init__(
         self,
-        session_factory=AiohttpSession,
-        api_key=None,
-        user_creds=None,
-        client_creds=None,
-        service_account_creds=None,
+        session_factory: Type[AbstractSession] = AiohttpSession,
+        api_key: Optional[ApiKey] = None,
+        user_creds: Optional[UserCreds] = None,
+        client_creds: Optional[ClientCreds] = None,
+        service_account_creds: Optional[ServiceAccountCreds] = None,
     ):
 
         self.session_factory = session_factory
@@ -91,9 +98,9 @@ class Aiogoogle:
         https://developers.google.com/discovery/v1/reference/apis/list
 
         The discovery.apis.list method returns the list all APIs supported by the Google APIs Discovery Service.
-        
+
         The data for each entry is a subset of the Discovery Document for that API, and the list provides a directory of supported APIs.
-        
+
         If a specific API has multiple versions, each of the versions has its own entry in the list.
 
         Example:
@@ -147,7 +154,7 @@ class Aiogoogle:
         )
         return await self.as_anon(request)
 
-    async def discover(self, api_name, api_version=None, validate=False, *, disco_doc_ver: Optional[int] = None):
+    async def discover(self, api_name: str, api_version: Optional[str] = None, validate: bool = False, *, disco_doc_ver: Optional[Literal[2]] = None) -> GoogleAPI:
         """
         Donwloads a discovery document from Google's Discovery Service V1 and sets it a ``aiogoogle.resource.GoogleAPI``
 
@@ -200,16 +207,25 @@ class Aiogoogle:
                 api=api_name, api_version=api_version
             )
 
-        discovery_document = await self.as_anon(request)
+        try:
+            discovery_document = await self.as_anon(request)
+
+        except Exception as e:
+            if isinstance(e, HTTPError):
+                if not disco_doc_ver:
+                    raise ValueError(
+                        f"Failed to fetch discovery document from v1 of the Google Discovery Service. Consider setting `disco_doc_ver=2` parmeter. Error: {e}."
+                    )
+            raise e
 
         return GoogleAPI(discovery_document, validate)
 
     # -------- Send Requests ----------#
 
     async def as_user(self, *requests, timeout=None, full_res=False, user_creds=None, raise_for_status=True):
-        """ 
+        """
         Sends requests on behalf of ``self.user_creds`` (OAuth2)
-        
+
         Arguments:
 
             *requests (aiogoogle.models.Request):
@@ -240,18 +256,12 @@ class Aiogoogle:
         if user_creds is None:
             raise TypeError("No user credentials were found")
 
-        # Refresh credentials
-        if (
-            user_creds.get("expires_at") is None
-            or (user_creds.get("expires_at") and self.oauth2.is_expired(user_creds) is True)
-        ):
-            user_creds = await self.oauth2.refresh(
-                user_creds, client_creds=self.client_creds
-            )
-
-            # Set refreshed user_creds if ones were already existing
-            if self.user_creds is not None:
-                self.user_creds = user_creds
+        is_refreshed, user_creds = await self.oauth2.refresh(
+            user_creds, client_creds=self.client_creds
+        )
+        # Set refreshed user_creds if ones already exist
+        if is_refreshed and self.user_creds is not None:
+            self.user_creds = user_creds
 
         authorized_requests = [
             self.oauth2.authorize(request, user_creds) for request in requests
@@ -263,14 +273,15 @@ class Aiogoogle:
             full_res=full_res,
             raise_for_status=raise_for_status,
             session_factory=self.session_factory,
-            auth_manager=self.oauth2
+            auth_manager=self.oauth2,
+            user_creds=user_creds
         )
 
     async def as_service_account(
-            self, *requests, timeout=None, full_res=False, service_account_creds=None, raise_for_status=True):
-        """ 
+            self, *requests: Request, timeout: Optional[int] = None, full_res: bool = False, service_account_creds: ServiceAccountCreds = None, raise_for_status: bool = True) -> Response:
+        """
         Sends requests on behalf of ``self.user_creds`` (OAuth2)
-        
+
         Arguments:
 
             *requests (aiogoogle.models.Request):
@@ -317,9 +328,9 @@ class Aiogoogle:
         )
 
     async def as_api_key(self, *requests, timeout=None, full_res=False, api_key=None, raise_for_status=True):
-        """ 
+        """
         Sends requests on behalf of ``self.api_key`` (OAuth2)
-        
+
         Arguments:
 
             *requests (aiogoogle.models.Request):
@@ -365,9 +376,9 @@ class Aiogoogle:
         )
 
     async def as_anon(self, *requests, timeout=None, full_res=False, raise_for_status=True):
-        """ 
+        """
         Sends unauthorized requests
-        
+
         Arguments:
 
             *requests (aiogoogle.models.Request):
@@ -413,7 +424,7 @@ class Aiogoogle:
             session = self._set_session()
         return await session.send(*args, **kwargs)
 
-    async def __aenter__(self):
+    async def __aenter__(self) -> Aiogoogle:
         session = self._get_session()
         if session is None:
             session = self._set_session()
@@ -421,7 +432,7 @@ class Aiogoogle:
             return self
         raise RuntimeError("Nesting context managers using the same Aiogoogle object is not allowed.")
 
-    async def __aexit__(self, *args):
+    async def __aexit__(self, *args: Any) -> None:
         session = self._get_session()
         await session.__aexit__(*args)
         # Had to add this because there's no use of keeping a closed session
