@@ -11,6 +11,9 @@ import async_timeout
 
 from ..models import Response
 from .abc import AbstractSession
+import os
+
+HTTP_PROXY = os.getenv("AIOGOOGLE_HTTP_PROXY", "")
 
 
 async def _get_file_size(full_file_path):
@@ -32,7 +35,19 @@ async def _read_file(file_name):
         return await file.read()
 
 
-class AiohttpSession(ClientSession, AbstractSession):
+class AiohttpSession(AbstractSession):
+    def __init__(self, *args, **kwargs):
+        self._session = ClientSession(*args, **kwargs)
+    
+    async def __aenter__(self):
+        return self
+    
+    async def __aexit__(self, exc_type, exc, tb):
+        await self._session.close()
+    
+    async def close(self):
+        await self._session.close()
+
     async def send(
         self,
         *requests,
@@ -119,38 +134,41 @@ class AiohttpSession(ClientSession, AbstractSession):
 
                         request.headers.update({"Content-Type": f"{req_content_type}; boundary={mpwriter.boundary}"})
 
-                        return await self.request(
+                        return await self._session.request(
                             method=request.method,
                             url=request.media_upload.upload_path,
                             headers=request.headers,
                             data=mpwriter,
                             timeout=request.timeout,
-                            verify_ssl=request._verify_ssl,
+                            ssl=request._verify_ssl,
+                            proxy=HTTP_PROXY,
                         )
                 # Else load file to memory and send
                 else:
                     read_file = await request.media_upload.read_file(_read_file)
                     if request.upload_file_content_type:
                         request.headers.update({"Content-Type": request.upload_file_content_type})
-                    return await self.request(
+                    return await self._session.request(
                         method=request.method,
                         url=request.media_upload.upload_path,
                         headers=request.headers,
                         data=read_file,
                         json=request.json,
                         timeout=request.timeout,
-                        verify_ssl=request._verify_ssl,
+                        ssl=request._verify_ssl,
+                        proxy=HTTP_PROXY
                     )
             # Else, if no file upload
             else:
-                return await self.request(
+                return await self._session.request(
                     method=request.method,
                     url=request.url,
                     headers=request.headers,
                     data=request.data,
                     json=request.json,
                     timeout=request.timeout,
-                    verify_ssl=request._verify_ssl,
+                    ssl=request._verify_ssl,
+                    proxy=HTTP_PROXY,
                 )
 
         # ----------------- send sequence ------------------#
@@ -168,15 +186,14 @@ class AiohttpSession(ClientSession, AbstractSession):
         # ----------------- /send sequence ------------------#
 
         async def schedule_tasks():
-            if full_res is True:
-                tasks = [
-                    asyncio.ensure_future(get_response(request)) for request in requests
-                ]
-            else:
-                tasks = [
-                    asyncio.ensure_future(get_content(request)) for request in requests
-                ]
-            return await asyncio.gather(*tasks, return_exceptions=False)
+            return await asyncio.gather(
+                *[
+                    asyncio.ensure_future(
+                        get_response(request) if full_res is True else get_content(request)
+                    ) for request in requests
+                ],
+                return_exceptions=False
+            )
 
         session_factory = self.__class__ if session_factory is None else session_factory
 
