@@ -297,7 +297,7 @@ def validate_type(instance, schema, schema_name=None):
     if type_validator_name not in JSON_PYTHON_TYPE_MAPPING:
         warnings.warn(
             f"""\n\nUnknown type: {type_validator_name} found.\n\nSkipping type checks for {instance}
-            against this schema:\n\n{schema}\n\nPlease create an 
+            against this schema:\n\n{schema}\n\nPlease create an
             issue @ https://github.com/omarryhan/aiogoogle and report this warning msg."""
         )
         return
@@ -316,7 +316,7 @@ def validate_format(instance, schema, schema_name=None):
         if format_validator_name not in KNOWN_FORMATS:
             warnings.warn(
                 f"""\n\nUnknown format: {format_validator_name} found.\n\nSkipping format checks for {instance}
-            against this schema:\n\n{schema}\n\nPlease create an 
+            against this schema:\n\n{schema}\n\nPlease create an
             issue @ https://github.com/omarryhan/aiogoogle and report this warning msg."""
             )
             return
@@ -329,7 +329,7 @@ def validate_format(instance, schema, schema_name=None):
 def validate_range(instance, schema, schema_name=None):
     if schema.get("minimum"):
         minimum_validator(instance, schema["minimum"], schema_name=schema_name)
-    elif schema.get("maximum"):
+    if schema.get("maximum"):
         maximum_validator(instance, schema["maximum"], schema_name=schema_name)
 
 
@@ -347,6 +347,18 @@ def validate_pattern(instance, schema, schema_name=None):
             )
 
 
+def validate_enum(instance, schema, schema_name=None):
+    options = schema.get("enum")
+    if options is not None and instance not in options:
+        option_string = ', '.join(f'"{o}"' for o in options)
+        raise ValidationError(
+            make_validation_error_msg(
+                instance,
+                f'Must be one of the following: {option_string}',
+                schema_name=schema_name
+            )
+        )
+
 # -- Main Validator ---------------
 
 
@@ -355,6 +367,7 @@ def validate_all(instance, schema, schema_name=None):
     validate_format(instance, schema, schema_name)
     validate_range(instance, schema, schema_name)
     validate_pattern(instance, schema, schema_name)
+    validate_enum(instance, schema, schema_name)
 
 
 # -- API --------------------
@@ -415,7 +428,7 @@ def validate(instance, schema, schemas=None, schema_name=None):
             if not additional_properties:
                 raise ValidationError(
                     f"""
-                    Invalid Schema: {str(schema)}. 
+                    Invalid Schema: {str(schema)}.
                     Neither properties nor addiotional properties found in this schema"""
                 )
             props = {}
